@@ -22,7 +22,7 @@ UmlDiagram * UmlBasePackage::associatedDiagram() {
 }
 
 bool UmlBasePackage::set_AssociatedDiagram(UmlDiagram * d) {
-  UmlCom::send_cmd(_identifier, setAssocDiagramCmd, ((UmlBaseItem *) d)->_identifier);
+  UmlCom::send_cmd(_identifier, setAssocDiagramCmd, (d == 0) ? (void *) 0 : ((UmlBaseItem *) d)->_identifier);
   if (UmlCom::read_bool()) {
     _assoc_diagram = d;
     return TRUE;
@@ -62,9 +62,9 @@ bool UmlBasePackage::set_CppNamespace(const QCString & s) {
   return set_it_(_cpp_namespace, s, setCppNamespaceCmd);
 }
 
-UmlPackage * UmlBasePackage::findNamespace(const QCString & n, const UmlBasePackage * p)
+UmlPackage * UmlBasePackage::findCppNamespace(const QCString & n, const UmlBasePackage * p)
 {
-  UmlCom::send_cmd(packageGlobalCmd, findNamespaceCmd, (p) ? p->_identifier : 0, n);
+  UmlCom::send_cmd(packageGlobalCmd, findCppNamespaceCmd, (p) ? p->_identifier : 0, n);
   
   return (UmlPackage *) UmlBaseItem::read_();  
 }
@@ -91,9 +91,9 @@ bool UmlBasePackage::set_JavaPackage(const QCString & s) {
   return set_it_(_java_package, s, setJavaPackageCmd);
 }
 
-UmlPackage * UmlBasePackage::findPackage(const QCString & n, const UmlBasePackage * p)
+UmlPackage * UmlBasePackage::findJavaPackage(const QCString & n, const UmlBasePackage * p)
 {
-  UmlCom::send_cmd(packageGlobalCmd, findPackageCmd, (p) ? p->_identifier : 0, n);
+  UmlCom::send_cmd(packageGlobalCmd, findJavaPackageCmd, (p) ? p->_identifier : 0, n);
   
   return (UmlPackage *) UmlBaseItem::read_();  
 }
@@ -111,6 +111,35 @@ bool UmlBasePackage::set_PhpDir(const QCString & s) {
 }
 #endif
 
+#ifdef WITHPYTHON
+const QCString & UmlBasePackage::pythonDir() {
+  read_if_needed_();
+  
+  return _python_dir;
+}
+
+bool UmlBasePackage::set_PythonDir(const QCString & s) {
+  return set_it_(_python_dir, s, setPythonDirCmd);
+}
+
+QCString UmlBasePackage::pythonPackage() {
+  read_if_needed_();
+  
+  return _python_package;
+}
+
+bool UmlBasePackage::set_PythonPackage(const QCString & s) {
+  return set_it_(_python_package, s, setPythonPackageCmd);
+}
+
+UmlPackage * UmlBasePackage::findPythonPackage(const QCString & n, const UmlBasePackage * p)
+{
+  UmlCom::send_cmd(packageGlobalCmd, findPythonPackageCmd, (p) ? p->_identifier : 0, n);
+  
+  return (UmlPackage *) UmlBaseItem::read_();  
+}
+#endif
+
 #ifdef WITHIDL
 const QCString & UmlBasePackage::idlDir() {
   read_if_needed_();
@@ -132,9 +161,9 @@ bool UmlBasePackage::set_IdlModule(const QCString & s) {
   return set_it_(_idl_module, s, setIdlModuleCmd);
 }
 
-UmlPackage * UmlBasePackage::findModule(const QCString & n, const UmlBasePackage * p)
+UmlPackage * UmlBasePackage::findIdlModule(const QCString & n, const UmlBasePackage * p)
 {
-  UmlCom::send_cmd(packageGlobalCmd, findModuleCmd, (p) ? p->_identifier : 0, n);
+  UmlCom::send_cmd(packageGlobalCmd, findIdlModuleCmd, (p) ? p->_identifier : 0, n);
   
   return (UmlPackage *) UmlBaseItem::read_();  
 }
@@ -164,6 +193,11 @@ void UmlBasePackage::loadProject(QCString p)
   UmlCom::send_cmd(miscGlobalCmd, loadCmd, (const char *) p);
 }
 
+void UmlBasePackage::updateProfiles()
+{
+  UmlCom::send_cmd(packageGlobalCmd, updateProfileCmd);
+}
+
 void UmlBasePackage::unload(bool rec, bool del) {
   _assoc_diagram = 0;
 #ifdef WITHCPP
@@ -178,6 +212,10 @@ void UmlBasePackage::unload(bool rec, bool del) {
 #ifdef WITHPHP
   _php_dir = 0;
 #endif
+#ifdef WITHPYTHON
+  _python_dir = 0;
+  _python_package = 0;
+#endif
 #ifdef WITHIDL
   _idl_dir = 0;
   _idl_module = 0;
@@ -211,6 +249,13 @@ void UmlBasePackage::read_php_() {
 }
 #endif
 
+#ifdef WITHPYTHON
+void UmlBasePackage::read_python_() {
+  _python_dir = UmlCom::read_string();
+  _python_package = UmlCom::read_string();
+}
+#endif
+
 #ifdef WITHIDL
 void UmlBasePackage::read_idl_() {
   _idl_dir = UmlCom::read_string();
