@@ -90,7 +90,7 @@ void FileOut::ref(UmlItem * x, const char * pfix1, const char * pfix2, int n) {
 }
 
 void FileOut::ref_only(UmlItem * x, const char * pfix) {
-  ((QTextStream &) *this) << pfix << ((void *) x->getIdentifier())
+  ((QTextStream &) *this) << "BOUML_" << pfix << ((void *) x->getIdentifier())
     << "_" << x->kind();
 }
 
