@@ -4,10 +4,8 @@
 
 #include "UmlBaseComponent.h"
 #include <qcstring.h>
-#include <qvector.h>
 
 class FileOut;
-class UmlClass;
 
 // This class manages 'components'
 //
@@ -18,8 +16,6 @@ class UmlComponent : public UmlBaseComponent {
 
     virtual void write(FileOut & out);
 
-    void write(FileOut & out, const QVector< UmlClass > & v, const char * k1, const char * k2);
-
 };
 
 #endif
