@@ -17,7 +17,7 @@ class UmlExpansionRegion : public UmlBaseExpansionRegion {
     }
 
     //returns the first container for a 'kind', going up in the browser tree
-    virtual UmlItem * container(anItemKind kind, const Token & token, FileIn & in);
+    virtual UmlItem * container(anItemKind kind, Token & token, FileIn & in);
 
     static void init();
 
