@@ -1,6 +1,6 @@
 // *************************************************************************
 //
-// Copyleft 2004-2008 Bruno PAGES  .
+// Copyleft 2004-2009 Bruno PAGES  .
 //
 // This file is part of the BOUML Uml Toolkit.
 //
@@ -303,6 +303,14 @@ void UmlOperation::generate(QTextOStream & f, const QCString &,
       p += 7;
       f << compute_name();
     }
+    else if (!strncmp(p, "${class}", 8)) {
+      if (indent_needed) {
+	indent_needed = FALSE;
+	f << indent;
+      }
+      p += 8;
+      f << parent()->name();
+    }
     else if (!strncmp(p, "${(}", 4)) {
       p += 4;
       f << '(';
@@ -340,6 +348,20 @@ void UmlOperation::generate(QTextOStream & f, const QCString &,
       isinline = FALSE;
       p = generate_body(f, indent, indent_needed, p);
     }
+    else if (!strncmp(p, "${association}", 14)) {
+      p += 14;
+      
+      UmlClassMember * m;
+      
+      if ((((m = getOf()) != 0) || ((m = setOf()) != 0)) &&
+	  (m->kind() == aRelation)) {
+	if (indent_needed) {
+	  indent_needed = FALSE;
+	  f << indent;
+	}
+	UmlClass::write(f, ((UmlRelation *) m)->association());
+      }
+    }
     else {
       // strange
       if (indent_needed) {
