@@ -210,6 +210,62 @@ class CppSettings : public UmlSettings {
     // On error : return FALSE in C++, produce a RuntimeException in Java
     static bool set_EnumReturn(QCString v);
 
+    // returns the default operation 'in' parameter specification
+    // in case its type is specified in the first 'Generation 
+    // settings' tab, else an empty string/null
+    
+    static QCString builtinIn(QCString s);
+
+    // set the default operation 'in' parameter specification
+    // in case its type is specified in the first 'Generation 
+    // settings' tab
+    //
+    // On error : return FALSE in C++, produce a RuntimeException in Java
+    
+    static bool set_BuiltinIn(QCString type, QCString form);
+
+    // returns the default operation 'out' parameter specification
+    // in case its type is specified in the first 'Generation 
+    // settings' tab, else an empty string/null
+    
+    static QCString builtinOut(QCString s);
+
+    // set the default operation 'out' parameter specification
+    // in case its type is specified in the first 'Generation 
+    // settings' tab
+    //
+    // On error : return FALSE in C++, produce a RuntimeException in Java
+    
+    static bool set_BuiltinOut(QCString type, QCString form);
+
+    // returns the default operation 'inout' parameter specification
+    // in case its type is specified in the first 'Generation 
+    // settings' tab, else an empty string/null
+    
+    static QCString builtinInOut(QCString s);
+
+    // set the default operation 'inout' parameter specification
+    // in case its type is specified in the first 'Generation 
+    // settings' tab
+    //
+    // On error : return FALSE in C++, produce a RuntimeException in Java
+    
+    static bool set_BuiltinInOut(QCString type, QCString form);
+
+    // returns the default operation 'return' parameter specification
+    // in case its type is specified in the first 'Generation 
+    // settings' tab, else an empty string/null
+    
+    static QCString builtinReturn(QCString s);
+
+    // set the default operation 'return' parameter specification
+    // in case its type is specified in the first 'Generation 
+    // settings' tab
+    //
+    // On error : return FALSE in C++, produce a RuntimeException in Java
+    
+    static bool set_BuiltinReturn(QCString type, QCString form);
+
     // returns the default operation 'in' parameter specification 
     // in case its type is not an enum or a type specified in the
     // first 'Generation settings' tab
@@ -551,8 +607,12 @@ class CppSettings : public UmlSettings {
 
 
   protected:
+    //internal, do NOT use it
+    
     static void read_();
 
+    //internal, do NOT use it
+    
     static void read_if_needed_();
 
 };
