@@ -58,7 +58,13 @@ bool UmlBaseRelation::isReadOnly() {
 }
 
 bool UmlBaseRelation::set_isReadOnly(bool y) {
-  return set_it_(_read_only, y, setIsReadOnlyCmd);
+  UmlCom::send_cmd(_identifier, setIsReadOnlyCmd, (char) y);
+  if (UmlCom::read_bool()) {
+    _read_only = y;
+    return TRUE;
+  }
+  else
+    return FALSE;
 }
 
 const QCString & UmlBaseRelation::defaultValue() {
@@ -283,6 +289,12 @@ void UmlBaseRelation::read_uml_() {
   _multiplicity = UmlCom::read_string();
   _default_value = UmlCom::read_string();
   _read_only = UmlCom::read_bool();
+#if APIVERSION > 41
+  /*_is_derived =*/ UmlCom::read_bool();
+  /*_is_derivedunion =*/ UmlCom::read_bool();
+  /*_is_ordered =*/ UmlCom::read_bool();
+  /*_is_unique =*/ UmlCom::read_bool();
+#endif
   _get_oper = (UmlOperation *) UmlBaseItem::read_();
   _set_oper = (UmlOperation *) UmlBaseItem::read_();
 }
