@@ -1,6 +1,6 @@
 // *************************************************************************
 //
-// Copyleft 2004-2008 Bruno PAGES  .
+// Copyleft 2004-2009 Bruno PAGES  .
 //
 // This file is part of the BOUML Uml Toolkit.
 //
@@ -44,8 +44,10 @@ class BrowserActivityDiagram : public BrowserDiagram {
     SimpleData * def;
     ActivityDiagramWindow * window;
     ActivityDiagramSettings settings;
+    ActivityDiagramSettings * used_settings;
     UmlColor activity_color;
     UmlColor activityregion_color;
+    UmlColor activitypartition_color;
     UmlColor activityaction_color;
     UmlColor parameterpin_color;
     UmlColor note_color;
@@ -79,18 +81,19 @@ class BrowserActivityDiagram : public BrowserDiagram {
     virtual const char * help_topic() const;
     virtual BasicData * get_data() const;
     virtual void set_name(const char * s);
+    virtual void update_drawing_settings();
     virtual void get_activitydiagramsettings(ActivityDiagramSettings &) const;
     virtual void get_activitydrawingsettings(ActivityDrawingSettings &) const;
-    virtual bool get_auto_label_position(UmlCode who) const;
-    virtual bool get_write_label_horizontally(UmlCode who) const;
-    virtual bool get_show_opaque_action_definition(UmlCode who) const;
+    virtual bool get_auto_label_position() const;
+    virtual bool get_write_label_horizontally() const;
+    virtual bool get_show_opaque_action_definition() const;
     virtual bool get_shadow() const;
     virtual bool get_draw_all_relations() const;
     virtual void dont_draw_all_relations();
-    virtual bool get_show_stereotype_properties(UmlCode k) const;
+    virtual bool get_show_stereotype_properties() const;
     virtual UmlColor get_color(UmlCode) const;
     virtual void package_settings(bool & name_in_tab, ShowContextMode & show_context) const;
-    virtual DrawingLanguage get_language(UmlCode who) const;
+    virtual DrawingLanguage get_language() const;
     virtual bool tool_cmd(ToolCom * com, const char * args);
     virtual void save(QTextStream &, bool ref, QString & warning);
     static BrowserActivityDiagram * read(char * &, char *, BrowserNode *);
@@ -107,6 +110,9 @@ class BrowserActivityDiagram : public BrowserDiagram {
     static void open_all();
     static void import();
     
+    static void compute_referenced_by(QList<BrowserNode> & l, BrowserNode *,
+				      const char * kc, char const * kr);
+    
     static QString drag_key(BrowserNode * p);
     virtual QString drag_key() const;
     virtual QString drag_postfix() const;
