@@ -1,6 +1,6 @@
 // *************************************************************************
 //
-// Copyleft 2004-2008 Bruno PAGES  .
+// Copyleft 2004-2009 Bruno PAGES  .
 //
 // This file is part of the BOUML Uml Toolkit.
 //
@@ -33,12 +33,14 @@
 
 #include "BrowserClassDiagram.h"
 #include "SimpleData.h"
+#include "RelationData.h"
 #include "BrowserClassView.h"
 #include "ClassDiagramWindow.h"
 #include "UmlPixmap.h"
 #include "SettingsDialog.h"
 #include "DiagramView.h"
 #include "myio.h"
+#include "strutil.h"
 #include "ToolCom.h"
 #include "Tool.h"
 #include "MenuTitle.h"
@@ -51,13 +53,13 @@ QValueList<int> BrowserClassDiagram::imported_ids;
 QStringList BrowserClassDiagram::its_default_stereotypes;	// unicode
 
 BrowserClassDiagram::BrowserClassDiagram(QString s, BrowserNode * p, int id)
-    : BrowserDiagram(s, p, id), window(0) {
+    : BrowserDiagram(s, p, id), window(0), used_settings(0) {
   make();
   is_modified = (id == 0);
 }
 
 BrowserClassDiagram::BrowserClassDiagram(BrowserClassDiagram * model, BrowserNode * p)
-    : BrowserDiagram(p->get_name(), p, 0), window(0) {
+    : BrowserDiagram(p->get_name(), p, 0), window(0), used_settings(0) {
   def = new SimpleData(model->def);
   def->set_browser_node(this);
   comment = model->comment;
@@ -82,7 +84,7 @@ BrowserClassDiagram::BrowserClassDiagram(BrowserClassDiagram * model, BrowserNod
 }
 
 BrowserClassDiagram::BrowserClassDiagram(int id)
-    : BrowserDiagram(id), window(0) {
+    : BrowserDiagram(id), window(0), used_settings(0) {
   // not yet read
   make();
   is_modified = (id == 0);
@@ -98,11 +100,6 @@ BrowserClassDiagram::~BrowserClassDiagram() {
     
     QFile::remove(d.absFilePath(fn));
   }
-#if 0
-  // already done before browser->clear
-  if (window)
-    delete window;
-#endif
   all.remove(get_ident());
   delete def;
 }
@@ -173,7 +170,12 @@ BrowserNode * BrowserClassDiagram::duplicate(BrowserNode * p, QString name) {
 }
 
 const QPixmap* BrowserClassDiagram::pixmap(int) const {
-  return (deletedp()) ? DeletedClassDiagramIcon : ClassDiagramIcon;
+  if (deletedp()) 
+    return DeletedClassDiagramIcon;
+  
+  const QPixmap * px = ProfiledStereotypes::browserPixmap(def->get_stereotype());
+
+  return (px != 0) ? px : ClassDiagramIcon;
 }
 
 void BrowserClassDiagram::draw_svg() const {
@@ -328,6 +330,10 @@ void BrowserClassDiagram::edit_settings() {
 
 void BrowserClassDiagram::on_close() {
   window = 0;
+  if (used_settings != 0) {
+    delete used_settings;
+    used_settings = 0;
+  }
 }
 
 void BrowserClassDiagram::read_session(char * & st) {
@@ -346,30 +352,25 @@ const char * BrowserClassDiagram::help_topic() const  {
   return "classdiagram";
 }
 
-void BrowserClassDiagram::get_classdiagramsettings(ClassDiagramSettings & r) const {
-  if (! settings.complete(r))
-    ((BrowserNode *) parent())->get_classdiagramsettings(r);
+void BrowserClassDiagram::update_drawing_settings() {
+  if (used_settings == 0)
+    used_settings = new ClassDiagramSettings;
+  *used_settings = settings;
+  ((BrowserNode *) parent())->get_classdiagramsettings(*used_settings);
 }
 
+void BrowserClassDiagram::get_classdiagramsettings(ClassDiagramSettings & r) const {
+  r.assign(*used_settings);
+}
 
 void BrowserClassDiagram::package_settings(bool & name_in_tab,
 					   ShowContextMode & show_context) const {
-  ClassDiagramSettings st;
-  
-  get_classdiagramsettings(st);
-  name_in_tab = st.package_name_in_tab == UmlYes;
-  show_context = st.show_context_mode;
+  name_in_tab = used_settings->package_name_in_tab == UmlYes;
+  show_context = used_settings->show_context_mode;
 }
 
-bool BrowserClassDiagram::get_auto_label_position(UmlCode who) const {
-  switch (settings.auto_label_position) {
-  case UmlYes:
-    return TRUE;
-  case UmlNo:
-    return FALSE;
-  default:
-    return ((BrowserNode *) parent())->get_auto_label_position(who);
-  }
+bool BrowserClassDiagram::get_auto_label_position() const {
+  return used_settings->auto_label_position == UmlYes;
 }
 
 UmlColor BrowserClassDiagram::get_color(UmlCode who) const {
@@ -395,40 +396,20 @@ UmlColor BrowserClassDiagram::get_color(UmlCode who) const {
 }
 
 bool BrowserClassDiagram::get_shadow() const {
-  switch (settings.shadow) {
-  case UmlYes:
-    return TRUE;
-  case UmlNo:
-    return FALSE;
-  default:
-    return ((BrowserNode *) parent())->get_shadow(UmlClassDiagram);
-  }  
+  return used_settings->shadow == UmlYes;
 }
 
 bool BrowserClassDiagram::get_draw_all_relations() const {
-  switch (settings.draw_all_relations) {
-  case UmlYes:
-    return TRUE;
-  case UmlNo:
-    return FALSE;
-  default:
-    return ((BrowserNode *) parent())->get_draw_all_relations(UmlClassDiagram);
-  }  
+  return used_settings->draw_all_relations == UmlYes;
 }
 
 void BrowserClassDiagram::dont_draw_all_relations() {
-  settings.draw_all_relations = UmlNo;
+  settings.draw_all_relations = 
+    used_settings->draw_all_relations = UmlNo;
 }
 
-bool BrowserClassDiagram::get_show_stereotype_properties(UmlCode) const {
-  switch (settings.show_stereotype_properties) {
-  case UmlYes:
-    return TRUE;
-  case UmlNo:
-    return FALSE;
-  default:
-    return ((BrowserNode *) parent())->get_show_stereotype_properties(UmlClassDiagram);
-  }
+bool BrowserClassDiagram::get_show_stereotype_properties() const {
+  return used_settings->show_stereotype_properties == UmlYes;
 }
 
 BasicData * BrowserClassDiagram::get_data() const {
@@ -467,6 +448,28 @@ bool BrowserClassDiagram::tool_cmd(ToolCom * com, const char * args) {
   }
 }
 
+void BrowserClassDiagram::compute_referenced_by(QList<BrowserNode> & l,
+						BrowserNode * bn,
+						char const * kc,
+						char const * kr)
+{
+  int id = (IsaRelation(bn->get_type()))
+    ? ((RelationData *) bn->get_data())->get_ident()
+    : bn->get_identifier();
+  IdIterator<BrowserDiagram> it(all);
+  BrowserDiagram * d;
+
+  while ((d = it.current()) != 0) {
+    if (!d->deletedp() && (d->get_type() == UmlClassDiagram)) {
+      if ((((BrowserClassDiagram *) d)->window != 0)
+	  ? ((BrowserClassDiagram *) d)->window->get_view()->is_present(bn)
+	  : is_referenced(read_definition(d->get_ident(), "diagram"), id, kc, kr))
+	l.append((BrowserClassDiagram *) d);
+    }
+    ++it;
+  }
+}
+
 void BrowserClassDiagram::save_stereotypes(QTextStream & st)
 {
   nl_indent(st);
@@ -571,7 +574,7 @@ BrowserClassDiagram * BrowserClassDiagram::read(char * & st, char * k,
     r->is_defined = TRUE;
 
     r->is_read_only = (!in_import() && read_only_file()) || 
-      (user_id() != 0) && r->is_api_base();
+      ((user_id() != 0) && r->is_api_base());
     
     QFileInfo fi(BrowserView::get_dir(), QString::number(id) + ".diagram");
     if (!in_import() && fi.exists() && !fi.isWritable())
