@@ -1,6 +1,6 @@
 // *************************************************************************
 //
-// Copyleft 2004-2008 Bruno PAGES  .
+// Copyleft 2004-2009 Bruno PAGES  .
 //
 // This file is part of the BOUML Uml Toolkit.
 //
@@ -111,7 +111,9 @@ class BrowserComponent : public BrowserNode, public Labeled<BrowserComponent> {
     virtual void renumber(int phase);
     
     virtual const QPixmap* pixmap (int) const;
+    virtual void iconChanged();
     
+    virtual void referenced_by(QList<BrowserNode> & l, bool ondelete);
     static void compute_referenced_by(QList<BrowserNode> &, BrowserClass *);
     
     static void init();
