@@ -1,6 +1,6 @@
 // *************************************************************************
 //
-// Copyleft 2004-2008 Bruno PAGES  .
+// Copyleft 2004-2009 Bruno PAGES  .
 //
 // This file is part of the BOUML Uml Toolkit.
 //
@@ -39,6 +39,7 @@
 #include "UmlPixmap.h"
 #include "SettingsDialog.h"
 #include "myio.h"
+#include "strutil.h"
 #include "ToolCom.h"
 #include "Tool.h"
 #include "MenuTitle.h"
@@ -51,20 +52,20 @@ QValueList<int> BrowserComponentDiagram::imported_ids;
 QStringList BrowserComponentDiagram::its_default_stereotypes;	// unicode
 
 BrowserComponentDiagram::BrowserComponentDiagram(QString s, BrowserNode * p, int id)
-    : BrowserDiagram(s, p, id), window(0) {
+    : BrowserDiagram(s, p, id), window(0), used_settings(0) {
   make();
   is_modified = (id == 0);
 }
 
 BrowserComponentDiagram::BrowserComponentDiagram(int id)
-    : BrowserDiagram(id), window(0) {
+    : BrowserDiagram(id), window(0), used_settings(0) {
   // not yet read
   make();
   is_modified = (id == 0);
 }
 
 BrowserComponentDiagram::BrowserComponentDiagram(BrowserComponentDiagram * model, BrowserNode * p)
-    : BrowserDiagram(p->get_name(), p, 0), window(0) {
+    : BrowserDiagram(p->get_name(), p, 0), window(0), used_settings(0) {
   def = new SimpleData(model->def);
   def->set_browser_node(this);
   comment = model->comment;
@@ -174,7 +175,12 @@ BrowserNode * BrowserComponentDiagram::duplicate(BrowserNode * p, QString name)
 }
 
 const QPixmap* BrowserComponentDiagram::pixmap(int) const {
-  return (deletedp()) ? DeletedComponentDiagramIcon : ComponentDiagramIcon;
+  if (deletedp()) 
+    return DeletedComponentDiagramIcon;
+  
+  const QPixmap * px = ProfiledStereotypes::browserPixmap(def->get_stereotype());
+
+  return (px != 0) ? px : ComponentDiagramIcon;
 }
 
 void BrowserComponentDiagram::draw_svg() const {
@@ -333,6 +339,10 @@ void BrowserComponentDiagram::edit_settings() {
 
 void BrowserComponentDiagram::on_close() {
   window = 0;
+  if (used_settings != 0) {
+    delete used_settings;
+    used_settings = 0;
+  }
 }
 
 void BrowserComponentDiagram::read_session(char * & st) {
@@ -351,18 +361,25 @@ const char * BrowserComponentDiagram::help_topic() const  {
   return "componentdiagram";
 }
 
+void BrowserComponentDiagram::update_drawing_settings() {
+  if (used_settings == 0)
+    used_settings = new ComponentDiagramSettings;
+  *used_settings = settings;
+  ((BrowserNode *) parent())->get_componentdiagramsettings(*used_settings);
+}
+
 void BrowserComponentDiagram::get_componentdiagramsettings(ComponentDiagramSettings & r) const {
-  if (! settings.complete(r))
-    ((BrowserNode *) parent())->get_componentdiagramsettings(r);
+  r.assign(*used_settings);
+}
+
+void BrowserComponentDiagram::get_componentdrawingsettings(ComponentDrawingSettings & r) const {
+  r.assign(*used_settings);
 }
 
 void BrowserComponentDiagram::package_settings(bool & name_in_tab,
 					       ShowContextMode & show_context) const {
-  ComponentDiagramSettings st;
-  
-  get_componentdiagramsettings(st);
-  name_in_tab = st.package_name_in_tab == UmlYes;
-  show_context = st.show_context_mode;
+  name_in_tab = used_settings->package_name_in_tab == UmlYes;
+  show_context = used_settings->show_context_mode;
 }
 
 UmlColor BrowserComponentDiagram::get_color(UmlCode who) const {
@@ -388,56 +405,25 @@ UmlColor BrowserComponentDiagram::get_color(UmlCode who) const {
 }
 
 bool BrowserComponentDiagram::get_shadow() const {
-  switch (settings.shadow) {
-  case UmlYes:
-    return TRUE;
-  case UmlNo:
-    return FALSE;
-  default:
-    return ((BrowserNode *) parent())->get_shadow(UmlComponentDiagram);
-  }  
+  return used_settings->shadow == UmlYes;
 }
 
 bool BrowserComponentDiagram::get_draw_all_relations() const {
-  switch (settings.draw_all_relations) {
-  case UmlYes:
-    return TRUE;
-  case UmlNo:
-    return FALSE;
-  default:
-    return ((BrowserNode *) parent())->get_draw_all_relations(UmlComponentDiagram);
-  }  
+  return used_settings->draw_all_relations == UmlYes;
 }
 
 void BrowserComponentDiagram::dont_draw_all_relations() {
-  settings.draw_all_relations = UmlNo;
+  settings.draw_all_relations = 
+    used_settings->draw_all_relations = UmlNo;
 }
 
-bool BrowserComponentDiagram::get_show_stereotype_properties(UmlCode) const {
-  switch (settings.componentdrawingsettings.show_stereotype_properties) {
-  case UmlYes:
-    return TRUE;
-  case UmlNo:
-    return FALSE;
-  default:
-    return ((BrowserNode *) parent())->get_show_stereotype_properties(UmlComponentDiagram);
-  }
+bool BrowserComponentDiagram::get_show_stereotype_properties() const {
+  return used_settings->componentdrawingsettings.show_stereotype_properties
+    == UmlYes;
 }
 
-bool BrowserComponentDiagram::get_auto_label_position(UmlCode who) const {
-  switch (settings.auto_label_position) {
-  case UmlYes:
-    return TRUE;
-  case UmlNo:
-    return FALSE;
-  default:
-    return ((BrowserNode *) parent())->get_auto_label_position(who);
-  }
-}
-
-void BrowserComponentDiagram::get_componentdrawingsettings(bool, ComponentDrawingSettings & result) const {
-  if (!settings.componentdrawingsettings.complete(result))
-    ((BrowserNode *) parent())->get_componentdrawingsettings(FALSE, result);
+bool BrowserComponentDiagram::get_auto_label_position() const {
+  return used_settings->auto_label_position == UmlYes;
 }
 
 BasicData * BrowserComponentDiagram::get_data() const {
@@ -476,6 +462,26 @@ bool BrowserComponentDiagram::tool_cmd(ToolCom * com, const char * args) {
   }
 }
 
+void BrowserComponentDiagram::compute_referenced_by(QList<BrowserNode> & l,
+						    BrowserNode * bn,
+						    char const * kc,
+						    char const * kr)
+{
+  int id = bn->get_identifier();
+  IdIterator<BrowserDiagram> it(all);
+  BrowserDiagram * d;
+
+  while ((d = it.current()) != 0) {
+    if (!d->deletedp() && (d->get_type() == UmlComponentDiagram)) {
+      if ((((BrowserComponentDiagram *) d)->window != 0)
+	  ? ((BrowserComponentDiagram *) d)->window->get_view()->is_present(bn)
+	  : is_referenced(read_definition(d->get_ident(), "diagram"), id, kc, kr))
+	l.append((BrowserComponentDiagram *) d);
+    }
+    ++it;
+  }
+}
+
 void BrowserComponentDiagram::save_stereotypes(QTextStream & st)
 {
   nl_indent(st);
@@ -581,8 +587,8 @@ BrowserComponentDiagram *
     
     r->is_defined = TRUE;
 
-    r->is_read_only = !in_import() && read_only_file() || 
-      (user_id() != 0) && r->is_api_base();
+    r->is_read_only = (!in_import() && read_only_file()) || 
+      ((user_id() != 0) && r->is_api_base());
     
     QFileInfo fi(BrowserView::get_dir(), QString::number(id) + ".diagram");
     if (!in_import() && fi.exists() && !fi.isWritable())
