@@ -1,6 +1,6 @@
 // *************************************************************************
 //
-// Copyleft 2004-2008 Bruno PAGES  .
+// Copyleft 2004-2009 Bruno PAGES  .
 //
 // This file is part of the BOUML Uml Toolkit.
 //
@@ -83,7 +83,10 @@ class BrowserDeploymentNode : public BrowserNode, public Labeled<BrowserDeployme
     static void update_idmax_for_root();
     virtual void renumber(int phase);
     
+    virtual void referenced_by(QList<BrowserNode> & l, bool ondelete);
+    
     virtual const QPixmap* pixmap (int) const;
+    virtual void iconChanged();
     
     static void init();
     static const QStringList & default_stereotypes();
