@@ -1,6 +1,6 @@
 // *************************************************************************
 //
-// Copyleft 2004-2008 Bruno PAGES  .
+// Copyleft 2004-2009 Bruno PAGES  .
 //
 // This file is part of the BOUML Uml Toolkit.
 //
@@ -34,17 +34,12 @@
 #include "BrowserInterruptibleActivityRegion.h"
 #include "SimpleData.h"
 #include "BasicDialog.h"
+#include "ReferenceDialog.h"
 #include "BrowserExpansionRegion.h"
 #include "BrowserActivityNode.h"
 #include "BrowserActivityAction.h"
 #include "BrowserActivityObject.h"
 #include "ActivityActionData.h"
-
-#warning
-
-/*
-#include "BrowserPartition.h"
-*/
 #include "BrowserActivityDiagram.h"
 #include "UmlDrag.h"
 #include "UmlPixmap.h"
@@ -103,6 +98,12 @@ void BrowserInterruptibleActivityRegion::update_idmax_for_root()
 {
   all.update_idmax_for_root();
 }
+
+void BrowserInterruptibleActivityRegion::referenced_by(QList<BrowserNode> & l, bool ondelete) {
+  BrowserNode::referenced_by(l, ondelete);
+  if (! ondelete)
+    BrowserActivityDiagram::compute_referenced_by(l, this, "interruptibleactivityregioncanvas", "interruptibleactivityregion_ref");
+}
     
 void BrowserInterruptibleActivityRegion::renumber(int phase) {
   if (phase != -1)
@@ -112,6 +113,11 @@ void BrowserInterruptibleActivityRegion::renumber(int phase) {
 const QPixmap* BrowserInterruptibleActivityRegion::pixmap(int) const {
   if (deletedp())
     return DeletedInterruptibleActivityRegionIcon;
+  
+  const QPixmap * px = ProfiledStereotypes::browserPixmap(def->get_stereotype());
+
+  if (px != 0)
+    return px;
   else if (is_marked)
     return InterruptibleActivityRegionMarkedIcon;
   else
@@ -191,19 +197,13 @@ void BrowserInterruptibleActivityRegion::menu() {
 		     "to add a nested <em>interruptible activity region</em>");
       m.setWhatsThis(m.insertItem("New expansion region", 1),
 		     "to add a nested <em>expansion region</em>");
-
-#warning partition
-
-      /*
-      m.setWhatsThis(m.insertItem("Add partition", 3),
-		     "to add a <em>Partition</em>");
-		     */
       m.setWhatsThis(m.insertItem("Add activity action", 6),
 		     "to add an <em>activity action</em> to the <em>region</em>");
       m.setWhatsThis(m.insertItem("Add object node", 7),
 		     "to add an <em>activity object node</em> to the <em>region</em>");
       m.insertSeparator();
     }
+  
     m.setWhatsThis(m.insertItem("Edit", 4),
 		   "to edit the <em>interruptible activity region</em>, \
 a double click with the left mouse button does the same thing");
@@ -222,6 +222,8 @@ a double click with the left mouse button does the same thing");
 		       "to delete the <em>interruptible activity region</em>. \
 Note that you can undelete it after");
     }
+    m.setWhatsThis(m.insertItem("Referenced by", 3),
+		   "to know who reference the <i>region</i>");
     mark_menu(m, "interruptible activity region", 90);
     ProfiledStereotypes::menu(m, this, 99990);
     if ((edition_number == 0) &&
@@ -257,13 +259,10 @@ void BrowserInterruptibleActivityRegion::exec_menu_choice(int rank) {
     BrowserExpansionRegion::add_expansionregion(this);
     break;
   case 3:
-
-#warning
-
-    //add_partition(this);
+    ReferenceDialog::show(this);
     return;
   case 4:
-    open(FALSE);
+    open(TRUE);
     return;
   case 5:
     {
@@ -331,21 +330,16 @@ void BrowserInterruptibleActivityRegion::apply_shortcut(QString s) {
 	choice = 0;
       else if (s == "New expansion region")
 	choice = 1;
-
-#warning partition
-
-      /*
-      m.setWhatsThis(m.insertItem("Add partition", 3),
-		     "to add a <em>Partition</em>");
-		     */
       else if (s == "Add activity action")
 	choice = 6;
       else if (s == "Add object node")
 	choice = 7;
     }
-    if (s == "Edit")
+    if (s == "Referenced by")
+      choice = 3;
+    else if (s == "Edit")
       choice = 4;
-    if (!is_read_only) {
+    else if (!is_read_only) {
       if (s == "Duplicate")
 	choice = 5;
       
@@ -480,14 +474,6 @@ bool BrowserInterruptibleActivityRegion::tool_cmd(ToolCom * com, const char * ar
 	case UmlActivityObject:
 	  (new BrowserActivityObject(args, this))->write_id(com);
 	  break;
-
-#warning
-
-	  /*
-	case UmlPartition:
-	  (BrowserRegion::add_partition(this, args))->write_id(com);
-	  break;
-	  */
 	default:
 	  if (IsaActivityNode(k))
 	    (new BrowserActivityNode(k, args, this))->write_id(com);
@@ -527,9 +513,6 @@ bool BrowserInterruptibleActivityRegion::may_contains_them(const QList<BrowserNo
   
   for (; it.current(); ++it) {
     switch (it.current()->get_type()) {
-    case UmlPartition:
-      duplicable = FALSE;
-      // no break !
     case UmlInterruptibleActivityRegion:
     case UmlExpansionRegion:
     case UmlActivityAction:
@@ -559,12 +542,6 @@ void BrowserInterruptibleActivityRegion::DragMoveEvent(QDragMoveEvent * e) {
   if (UmlDrag::canDecode(e, BrowserActivityNode::drag_key(this)) ||
       UmlDrag::canDecode(e, BrowserActivityAction::drag_key(this)) ||
       UmlDrag::canDecode(e, BrowserExpansionRegion::drag_key(this)) ||
-
-#warning
-
-  /*
-      UmlDrag::canDecode(e, BrowserPartition::drag_key(this)) ||
-      */
       UmlDrag::canDecode(e, BrowserInterruptibleActivityRegion::drag_key(this))) {
     if (!is_read_only)
       e->accept();
@@ -584,12 +561,6 @@ void BrowserInterruptibleActivityRegion::DragMoveInsideEvent(QDragMoveEvent * e)
       (UmlDrag::canDecode(e, BrowserActivityNode::drag_key(this)) ||
        UmlDrag::canDecode(e, BrowserActivityAction::drag_key(this)) ||
        UmlDrag::canDecode(e, BrowserExpansionRegion::drag_key(this)) ||
-
-#warning
-
-  /*
-       UmlDrag::canDecode(e, BrowserPartition::drag_key(this)) ||
-      */
        UmlDrag::canDecode(e, BrowserInterruptibleActivityRegion::drag_key(this))))
     e->accept();
   else
@@ -602,12 +573,6 @@ void BrowserInterruptibleActivityRegion::DropAfterEvent(QDropEvent * e, BrowserN
   if ((((bn = UmlDrag::decode(e, BrowserActivityNode::drag_key(this))) != 0) ||
        ((bn = UmlDrag::decode(e, BrowserActivityAction::drag_key(this))) != 0) ||
        ((bn = UmlDrag::decode(e, BrowserExpansionRegion::drag_key(this))) != 0) ||
-
-#warning
-
-  /*
-       ((bn = UmlDrag::decode(e, BrowserPartition::drag_key(this))) != 0) ||
-       */
        ((bn = UmlDrag::decode(e, BrowserInterruptibleActivityRegion::drag_key(this))) != 0)) &&
       (bn != after) && (bn != this)) {
     if (may_contains(bn, FALSE)) 
@@ -767,20 +732,14 @@ BrowserInterruptibleActivityRegion *
     
     result->BrowserNode::read(st, k);
     
-    result->is_read_only = !in_import() && read_only_file() || 
-      (user_id() != 0) && result->is_api_base();
+    result->is_read_only = (!in_import() && read_only_file()) || 
+      ((user_id() != 0) && result->is_api_base());
     
     if (strcmp(k, "end")) {
       while (BrowserActivityNode::read(st, k, result) ||
 	     BrowserActivityAction::read(st, k, result) ||
 	     BrowserActivityObject::read(st, k, result) ||
 	     BrowserExpansionRegion::read(st, k, result) ||
-
-#warning
-
-	     /*
-	     BrowserPartition::read(st, k, result) ||
-	     */
 	     BrowserInterruptibleActivityRegion::read(st, k, result))
 	k = read_keyword(st);
       if (strcmp(k, "end"))
