@@ -1,6 +1,6 @@
 // *************************************************************************
 //
-// Copyleft 2004-2008 Bruno PAGES  .
+// Copyleft 2004-2009 Bruno PAGES  .
 //
 // This file is part of the BOUML Uml Toolkit.
 //
@@ -34,7 +34,9 @@
 #include "BrowserPin.h"
 #include "PinData.h"
 #include "BrowserActivityAction.h"
+#include "BrowserActivityDiagram.h"
 #include "ActivityActionData.h"
+#include "ReferenceDialog.h"
 #include "BrowserFlow.h"
 #include "BrowserClass.h"
 #include "ParameterData.h"
@@ -102,11 +104,22 @@ void BrowserPin::renumber(int phase) {
 const QPixmap* BrowserPin::pixmap(int) const {
   if (deletedp())
     return DeletedPinIcon;
+  
+  const QPixmap * px = ProfiledStereotypes::browserPixmap(def->get_stereotype());
+
+  if (px != 0)
+    return px;
   else if (markedp())
     return PinMarkedIcon;
   else
     return PinIcon;
 }
+    
+void BrowserPin::referenced_by(QList<BrowserNode> & l, bool ondelete) {
+  BrowserNode::referenced_by(l, ondelete);
+  if (! ondelete)
+    BrowserActivityDiagram::compute_referenced_by(l, this, "pincanvas", "pin_ref");
+}
 
 void BrowserPin::compute_referenced_by(QList<BrowserNode> & l,
 				       BrowserNode * target)
@@ -222,8 +235,8 @@ void BrowserPin::menu() {
   m.insertSeparator();
   if (!deletedp()) {
     if (!is_edited)
-    m.setWhatsThis(m.insertItem("Edit", 0),
-		   "to edit the <em>pin</em>, \
+      m.setWhatsThis(m.insertItem("Edit", 0),
+		     "to edit the <em>pin</em>, \
 a double click with the left mouse button does the same thing");
     if (!is_read_only && (edition_number == 0)) {
       if (((ActivityActionData *) ((BrowserNode *) parent())->get_data())->may_add_pin()) {
@@ -235,6 +248,8 @@ a double click with the left mouse button does the same thing");
 		     "to delete the <em>pin</em>. \
 Note that you can undelete it after");
     }
+    m.setWhatsThis(m.insertItem("Referenced by", 4),
+		   "to know who reference the <i>pin</i>");
     mark_menu(m, "pin", 90);
     ProfiledStereotypes::menu(m, this, 99990);
     if ((edition_number == 0) &&
@@ -255,7 +270,7 @@ Note that you can undelete it after");
 void BrowserPin::exec_menu_choice(int rank) {
   switch (rank) {
   case 0:
-    open(FALSE);
+    open(TRUE);
     return;
   case 1:
     add_pin(this, (BrowserNode *) parent());
@@ -266,6 +281,9 @@ void BrowserPin::exec_menu_choice(int rank) {
   case 3:
     undelete(FALSE);
     break;
+  case 4:
+    ReferenceDialog::show(this);
+    return;
   default:
     if (rank >= 99990)
       ProfiledStereotypes::choiceManagement(this, rank - 99990);
@@ -286,6 +304,8 @@ void BrowserPin::apply_shortcut(QString s) {
     if (!is_edited)
       if (s == "Edit")
 	choice = 0;
+    if (s == "Referenced by")
+      choice = 4;
     if (!is_read_only && (edition_number == 0)) {
       if (((ActivityActionData *) ((BrowserNode *) parent())->get_data())->may_add_pin()) {
 	if (s == "Duplicate")
@@ -534,7 +554,7 @@ BrowserPin * BrowserPin::read(char * & st, char * k, BrowserNode * parent)
     }
     
     result->is_read_only = (!in_import() && read_only_file()) || 
-      (user_id() != 0) && result->is_api_base();
+      ((user_id() != 0) && result->is_api_base());
     result->def->set_browser_node(result);
     
     if (strcmp(k, "end"))
