@@ -1,6 +1,6 @@
 // *************************************************************************
 //
-// Copyleft 2004-2008 Bruno PAGES  .
+// Copyleft 2004-2009 Bruno PAGES  .
 //
 // This file is part of the BOUML Uml Toolkit.
 //
@@ -85,6 +85,7 @@ BrowserUseCase::BrowserUseCase(const BrowserUseCase * model,
   stateaction_color = model->stateaction_color;
   activity_color = model->activity_color;
   activityregion_color = model->activityregion_color;
+  activitypartition_color = model->activitypartition_color;
   activityaction_color = model->activityaction_color;
   parameterpin_color = model->parameterpin_color;
   class_color = model->class_color;
@@ -120,6 +121,7 @@ void BrowserUseCase::make() {
   stateaction_color = UmlDefaultColor;
   activity_color = UmlDefaultColor;
   activityregion_color = UmlDefaultColor;
+  activitypartition_color = UmlDefaultColor;
   activityaction_color = UmlDefaultColor;
   parameterpin_color = UmlDefaultColor;   
   class_color = UmlDefaultColor;   
@@ -146,6 +148,12 @@ void BrowserUseCase::update_idmax_for_root()
   BrowserSimpleRelation::update_idmax_for_root();
 }
     
+void BrowserUseCase::referenced_by(QList<BrowserNode> & l, bool ondelete) {
+  BrowserNode::referenced_by(l, ondelete);
+  if (! ondelete)
+    BrowserUseCaseDiagram::compute_referenced_by(l, this, "usecasecanvas", "usecase_ref");
+}
+    
 void BrowserUseCase::renumber(int phase) {
   if (phase != -1)
     new_ident(phase, all);
@@ -153,7 +161,12 @@ void BrowserUseCase::renumber(int phase) {
 }
 
 const QPixmap* BrowserUseCase::pixmap(int) const {
-  return (deletedp()) ? DeletedUseCaseIcon : UseCaseIcon;
+  if (deletedp())
+    return DeletedUseCaseIcon;
+  
+  const QPixmap * px = ProfiledStereotypes::browserPixmap(def->get_stereotype());
+
+  return (px != 0) ? px : UseCaseIcon;
 }
 
 QString BrowserUseCase::full_name(bool rev, bool) const {
@@ -294,7 +307,7 @@ void BrowserUseCase::exec_menu_choice(int rank) {
   case 7:
     {
       QArray<StateSpec> st;
-      QArray<ColorSpec> co(14);
+      QArray<ColorSpec> co(15);
       
       usecasediagram_settings.complete(st, FALSE);
       sequencediagram_settings.complete(st, FALSE);
@@ -314,9 +327,10 @@ void BrowserUseCase::exec_menu_choice(int rank) {
       co[8].set("state action color", &stateaction_color);
       co[9].set("activity color", &activity_color);
       co[10].set("activity region color", &activityregion_color);
-      co[11].set("activity action color", &activityaction_color);
-      co[12].set("parameter and pin color", &parameterpin_color);
-      co[13].set("class color", &class_color);
+      co[11].set("activity partition color", &activitypartition_color);
+      co[12].set("activity action color", &activityaction_color);
+      co[13].set("parameter and pin color", &parameterpin_color);
+      co[14].set("class color", &class_color);
 
       SettingsDialog dialog(&st, &co, FALSE, FALSE);
       
@@ -548,11 +562,6 @@ void BrowserUseCase::get_activitydiagramsettings(ActivityDiagramSettings & r) co
     ((BrowserNode *) parent())->get_activitydiagramsettings(r);
 }
 
-void BrowserUseCase::get_simpleclassdiagramsettings(SimpleClassDiagramSettings & r) const {
-  if (!usecasediagram_settings.complete(r))
-    ((BrowserNode *) parent())->get_simpleclassdiagramsettings(r);
-}
-
 UmlColor BrowserUseCase::get_color(UmlCode who) const {
   UmlColor c;
   
@@ -588,6 +597,9 @@ UmlColor BrowserUseCase::get_color(UmlCode who) const {
   case UmlExpansionRegion:
     c = activityregion_color;
     break;
+  case UmlActivityPartition:
+    c = activitypartition_color;
+    break;
   case UmlActivityAction:
     c = activityaction_color;
     break;
@@ -608,128 +620,6 @@ UmlColor BrowserUseCase::get_color(UmlCode who) const {
     : ((BrowserNode *) parent())->get_color(who);
 }
 
-bool BrowserUseCase::get_auto_label_position(UmlCode who) const {
-  Uml3States v;
-  
-  switch (who) {
-  case UmlUseCaseDiagram:
-    v = usecasediagram_settings.auto_label_position;
-    break;
-  case UmlObjectDiagram:
-    v = objectdiagram_settings.auto_label_position;
-    break;
-  case UmlStateDiagram:
-    v = statediagram_settings.auto_label_position;
-    break;
-  case UmlActivityDiagram:
-    v = activitydiagram_settings.auto_label_position;
-    break;
-  default:
-    // error
-    return FALSE;
-  }
-  
-  switch (v) {
-  case UmlYes:
-    return TRUE;
-  case UmlNo:
-    return FALSE;
-  default:
-    return ((BrowserNode *) parent())->get_auto_label_position(who);
-  }
-}
-
-
-
-bool BrowserUseCase::get_write_label_horizontally(UmlCode who) const {
-  Uml3States v;
-  
-  switch (who) {
-  case UmlStateDiagram:
-    v = statediagram_settings.write_label_horizontally;
-    break;
-  case UmlActivityDiagram:
-    v = activitydiagram_settings.write_label_horizontally;
-    break;
-  default:
-    // error
-    return FALSE;
-  }
-  
-  switch (v) {
-  case UmlYes:
-    return TRUE;
-  case UmlNo:
-    return FALSE;
-  default:
-    return ((BrowserNode *) parent())->get_write_label_horizontally(who);
-  }
-}
-
-bool BrowserUseCase::get_show_trans_definition(UmlCode who) const {
-  Uml3States v;
-  
-  switch (who) {
-  case UmlStateDiagram:
-    v = statediagram_settings.show_trans_definition;
-    break;
-  default:
-    // error
-    return FALSE;
-  }
-  
-  switch (v) {
-  case UmlYes:
-    return TRUE;
-  case UmlNo:
-    return FALSE;
-  default:
-    return ((BrowserNode *) parent())->get_show_trans_definition(who);
-  }
-}
-
-bool BrowserUseCase::get_show_opaque_action_definition(UmlCode who) const {
-  Uml3States v;
-  
-  switch (who) {
-  case UmlActivityDiagram:
-    v = activitydiagram_settings.show_opaque_action_definition;
-    break;
-  default:
-    // error
-    return FALSE;
-  }
-  
-  switch (v) {
-  case UmlYes:
-    return TRUE;
-  case UmlNo:
-    return FALSE;
-  default:
-    return ((BrowserNode *) parent())->get_show_opaque_action_definition(who);
-  }
-}
-
-DrawingLanguage BrowserUseCase::get_language(UmlCode who) const {
-  DrawingLanguage v;
-  
-  switch (who) {
-  case UmlStateDiagram:
-    v = statediagram_settings.statedrawingsettings.drawing_language;
-    break;
-  case UmlActivityDiagram:
-    v = activitydiagram_settings.activitydrawingsettings.drawing_language;
-    break;
-  default:
-    // error
-    return UmlView;
-  }
-  
-  return (v != DefaultDrawingLanguage)
-    ? v
-    : ((BrowserNode *) parent())->get_language(who);
-}
-
 BrowserNode * BrowserUseCase::get_associated() const {
   return associated_diagram;
 }
@@ -870,14 +760,17 @@ bool BrowserUseCase::tool_cmd(ToolCom * com, const char * args) {
 }
 
 void BrowserUseCase::DragMoveEvent(QDragMoveEvent * e) {
-  if (UmlDrag::canDecode(e, UmlUseCaseDiagram) ||
+  if (UmlDrag::canDecode(e, UmlUseCase) ||
+      UmlDrag::canDecode(e, UmlUseCaseDiagram) ||
       UmlDrag::canDecode(e, UmlSeqDiagram) ||
       UmlDrag::canDecode(e, UmlColDiagram) ||
       UmlDrag::canDecode(e, UmlObjectDiagram) ||
       UmlDrag::canDecode(e, UmlState) ||
       UmlDrag::canDecode(e, UmlActivity) ||
       UmlDrag::canDecode(e, BrowserSimpleRelation::drag_key(this))) {
-    if (!is_read_only)
+    if (!is_read_only ||
+	(((BrowserNode *) parent())->is_writable() &&
+	 (((BrowserNode *) parent())->get_type() != UmlPackage)))
       e->accept();
     else
       e->ignore();
@@ -953,12 +846,14 @@ void BrowserUseCase::DropAfterEvent(QDropEvent * e, BrowserNode * after) {
        ((bn = UmlDrag::decode(e, UmlState)) != 0) ||
        ((bn = UmlDrag::decode(e, UmlActivity)) != 0)) &&
       (bn != after) && (bn != this)) {
-    if (may_contains(bn, bn->get_type() == UmlUseCase)) {
+    bool uc = bn->get_type() == UmlUseCase;
+    
+    if (may_contains(bn, uc)) {
       BrowserNode * x = this;
       
       if ((after == 0) &&
-	  (bn->get_type() == UmlUseCase) &&
-	  ((BrowserNode *) parent())->may_contains(bn, TRUE)) {
+	  (((BrowserNode *) parent())->get_type() != UmlPackage) &&
+	  ((BrowserNode *) parent())->may_contains(bn, uc)) {
 	// have choice
 	QPopupMenu m(0);
   
@@ -1118,6 +1013,7 @@ void BrowserUseCase::save(QTextStream & st, bool ref, QString & warning) {
     save_color(st, "stateaction_color", stateaction_color, nl);
     save_color(st, "activity_color", activity_color, nl);
     save_color(st, "activityregion_color", activityregion_color, nl);
+    save_color(st, "activitypartition_color", activitypartition_color, nl);
     save_color(st, "activityaction_color", activityaction_color, nl);
     save_color(st, "parameterpin_color", parameterpin_color, nl);
     save_color(st, "class_color", class_color, nl);
@@ -1206,8 +1102,8 @@ BrowserUseCase * BrowserUseCase::read(char * & st, char * k,
     
     result->is_defined = TRUE;
 
-    result->is_read_only = !in_import() && read_only_file() || 
-      (user_id() != 0) && result->is_api_base();
+    result->is_read_only = (!in_import() && read_only_file()) || 
+      ((user_id() != 0) && result->is_api_base());
     
     k = read_keyword(st);    
     result->def->read(st, k);				// updates k
@@ -1233,6 +1129,7 @@ BrowserUseCase * BrowserUseCase::read(char * & st, char * k,
       read_color(st, "stateaction_color", result->stateaction_color, k);		// updates k
       read_color(st, "activity_color", result->activity_color, k);		// updates k
       read_color(st, "activityregion_color", result->activityregion_color, k);		// updates k
+      read_color(st, "activitypartition_color", result->activitypartition_color, k);		// updates k
       read_color(st, "activityaction_color", result->activityaction_color, k);		// updates k
       read_color(st, "parameterpin_color", result->parameterpin_color, k);		// updates k
     }    
