@@ -1,6 +1,6 @@
 // *************************************************************************
 //
-// Copyleft 2004-2008 Bruno PAGES  .
+// Copyleft 2004-2009 Bruno PAGES  .
 //
 // This file is part of the BOUML Uml Toolkit.
 //
@@ -32,6 +32,7 @@
 #include <qfileinfo.h>
 
 #include "BrowserUseCaseDiagram.h"
+#include "RelationData.h"
 #include "SimpleData.h"
 #include "UseCaseDiagramWindow.h"
 #include "BrowserUseCaseView.h"
@@ -39,6 +40,7 @@
 #include "UmlPixmap.h"
 #include "SettingsDialog.h"
 #include "myio.h"
+#include "strutil.h"
 #include "ToolCom.h"
 #include "Tool.h"
 #include "MenuTitle.h"
@@ -51,20 +53,20 @@ QValueList<int> BrowserUseCaseDiagram::imported_ids;
 QStringList BrowserUseCaseDiagram::its_default_stereotypes;	// unicode
 
 BrowserUseCaseDiagram::BrowserUseCaseDiagram(QString s, BrowserNode * p, int id)
-    : BrowserDiagram(s, p, id), window(0) {
+    : BrowserDiagram(s, p, id), window(0), used_settings(0) {
   make();
   is_modified = (id == 0);
 }
 
 BrowserUseCaseDiagram::BrowserUseCaseDiagram(int id)
-    : BrowserDiagram(id), window(0) {
+    : BrowserDiagram(id), window(0), used_settings(0) {
   // not yet read
   make();
   is_modified = (id == 0);
 }
 
 BrowserUseCaseDiagram::BrowserUseCaseDiagram(BrowserUseCaseDiagram * model, BrowserNode * p)
-    : BrowserDiagram(p->get_name(), p, 0), window(0) {
+    : BrowserDiagram(p->get_name(), p, 0), window(0), used_settings(0) {
   def = new SimpleData(model->def);
   def->set_browser_node(this);
   comment = model->comment;
@@ -178,7 +180,12 @@ BrowserNode * BrowserUseCaseDiagram::duplicate(BrowserNode * p, QString name) {
 }
 
 const QPixmap* BrowserUseCaseDiagram::pixmap(int) const {
-  return (deletedp()) ? DeletedUseCaseDiagramIcon : UseCaseDiagramIcon;
+  if (deletedp()) 
+    return DeletedUseCaseDiagramIcon;
+  
+  const QPixmap * px = ProfiledStereotypes::browserPixmap(def->get_stereotype());
+
+  return (px != 0) ? px : UseCaseDiagramIcon;
 }
 
 void BrowserUseCaseDiagram::draw_svg() const {
@@ -339,6 +346,10 @@ void BrowserUseCaseDiagram::edit_settings() {
 
 void BrowserUseCaseDiagram::on_close() {
   window = 0;
+  if (used_settings != 0) {
+    delete used_settings;
+    used_settings = 0;
+  }
 }
 
 void BrowserUseCaseDiagram::read_session(char * & st) {
@@ -357,23 +368,25 @@ const char * BrowserUseCaseDiagram::help_topic() const  {
   return "usecasediagram";
 }
 
+void BrowserUseCaseDiagram::update_drawing_settings() {
+  if (used_settings == 0)
+    used_settings = new UseCaseDiagramSettings;
+  *used_settings = settings;
+  ((BrowserNode *) parent())->get_usecasediagramsettings(*used_settings);
+}
+
 void BrowserUseCaseDiagram::get_usecasediagramsettings(UseCaseDiagramSettings & r) const {
-  if (! settings.complete(r))
-    ((BrowserNode *) parent())->get_usecasediagramsettings(r);
+  r.assign(*used_settings);
 }
 
 void BrowserUseCaseDiagram::get_simpleclassdiagramsettings(SimpleClassDiagramSettings & r) const {
-  if (!settings.complete(r))
-    ((BrowserNode *) parent())->get_simpleclassdiagramsettings(r);
+  r.assign(*used_settings);
 }
 
 void BrowserUseCaseDiagram::package_settings(bool & name_in_tab,
 					     ShowContextMode & show_context) const {
-  UseCaseDiagramSettings st;
-  
-  get_usecasediagramsettings(st);
-  name_in_tab = st.package_name_in_tab == UmlYes;
-  show_context = st.show_context_mode;
+  name_in_tab = used_settings->package_name_in_tab == UmlYes;
+  show_context = used_settings->show_context_mode;
 }
 
 UmlColor BrowserUseCaseDiagram::get_color(UmlCode who) const {
@@ -405,51 +418,24 @@ UmlColor BrowserUseCaseDiagram::get_color(UmlCode who) const {
 }
 
 bool BrowserUseCaseDiagram::get_shadow() const {
-  switch (settings.shadow) {
-  case UmlYes:
-    return TRUE;
-  case UmlNo:
-    return FALSE;
-  default:
-    return ((BrowserNode *) parent())->get_shadow(UmlUseCaseDiagram);
-  }  
+  return used_settings->shadow == UmlYes;
 }
 
 bool BrowserUseCaseDiagram::get_draw_all_relations() const {
-  switch (settings.draw_all_relations) {
-  case UmlYes:
-    return TRUE;
-  case UmlNo:
-    return FALSE;
-  default:
-    return ((BrowserNode *) parent())->get_draw_all_relations(UmlUseCaseDiagram);
-  }  
+  return used_settings->draw_all_relations == UmlYes;
 }
 
 void BrowserUseCaseDiagram::dont_draw_all_relations() {
-  settings.draw_all_relations = UmlNo;
+  settings.draw_all_relations = 
+    used_settings->draw_all_relations = UmlNo;
 }
 
-bool BrowserUseCaseDiagram::get_show_stereotype_properties(UmlCode) const {
-  switch (settings.show_stereotype_properties) {
-  case UmlYes:
-    return TRUE;
-  case UmlNo:
-    return FALSE;
-  default:
-    return ((BrowserNode *) parent())->get_show_stereotype_properties(UmlUseCaseDiagram);
-  }
+bool BrowserUseCaseDiagram::get_show_stereotype_properties() const {
+  return used_settings->show_stereotype_properties == UmlYes;
 }
 
-bool BrowserUseCaseDiagram::get_auto_label_position(UmlCode who) const {
-  switch (settings.auto_label_position) {
-  case UmlYes:
-    return TRUE;
-  case UmlNo:
-    return FALSE;
-  default:
-    return ((BrowserNode *) parent())->get_auto_label_position(who);
-  }
+bool BrowserUseCaseDiagram::get_auto_label_position() const {
+  return used_settings->auto_label_position == UmlYes;
 }
 
 BasicData * BrowserUseCaseDiagram::get_data() const {
@@ -499,6 +485,28 @@ bool BrowserUseCaseDiagram::tool_cmd(ToolCom * com, const char * args) {
   }
 }
 
+void BrowserUseCaseDiagram::compute_referenced_by(QList<BrowserNode> & l,
+						  BrowserNode * bn,
+						  char const * kc,
+						  char const * kr)
+{
+  int id = (IsaRelation(bn->get_type()))
+    ? ((RelationData *) bn->get_data())->get_ident()
+    : bn->get_identifier();
+  IdIterator<BrowserDiagram> it(all);
+  BrowserDiagram * d;
+
+  while ((d = it.current()) != 0) {
+    if (!d->deletedp() && (d->get_type() == UmlUseCaseDiagram)) {
+      if ((((BrowserUseCaseDiagram *) d)->window != 0)
+	  ? ((BrowserUseCaseDiagram *) d)->window->get_view()->is_present(bn)
+	  : is_referenced(read_definition(d->get_ident(), "diagram"), id, kc, kr))
+	l.append((BrowserUseCaseDiagram *) d);
+    }
+    ++it;
+  }
+}
+
 void BrowserUseCaseDiagram::save_stereotypes(QTextStream & st)
 {
   nl_indent(st);
@@ -606,8 +614,8 @@ BrowserUseCaseDiagram * BrowserUseCaseDiagram::read(char * & st, char * k,
     
     r->is_defined = TRUE;
 
-    r->is_read_only = !in_import() && read_only_file() || 
-      (user_id() != 0) && r->is_api_base();
+    r->is_read_only = (!in_import() && read_only_file()) || 
+      ((user_id() != 0) && r->is_api_base());
     
     QFileInfo fi(BrowserView::get_dir(), QString::number(id) + ".diagram");
     if (!in_import() && fi.exists() && !fi.isWritable())
