@@ -1,6 +1,6 @@
 // *************************************************************************
 //
-// Copyleft 2004-2008 Bruno PAGES  .
+// Copyleft 2004-2009 Bruno PAGES  .
 //
 // This file is part of the BOUML Uml Toolkit.
 //
@@ -44,6 +44,7 @@ class BrowserUseCaseDiagram : public BrowserDiagram {
     SimpleData * def;
     UseCaseDiagramWindow * window;
     UseCaseDiagramSettings settings;
+    UseCaseDiagramSettings * used_settings;
     UmlColor usecase_color;
     UmlColor note_color;
     UmlColor fragment_color;
@@ -81,10 +82,11 @@ class BrowserUseCaseDiagram : public BrowserDiagram {
     virtual bool get_shadow() const;
     virtual bool get_draw_all_relations() const;
     virtual void dont_draw_all_relations();
-    virtual bool get_show_stereotype_properties(UmlCode k) const;
+    virtual bool get_show_stereotype_properties() const;
+    virtual void update_drawing_settings();
     virtual void get_usecasediagramsettings(UseCaseDiagramSettings & r) const;
     virtual void get_simpleclassdiagramsettings(SimpleClassDiagramSettings & r) const;
-    virtual bool get_auto_label_position(UmlCode who) const ;    
+    virtual bool get_auto_label_position() const;    
     virtual void package_settings(bool & name_in_tab, ShowContextMode & show_context) const;
     virtual bool tool_cmd(ToolCom * com, const char * args);
     virtual void save(QTextStream &, bool ref, QString & warning);
@@ -103,6 +105,9 @@ class BrowserUseCaseDiagram : public BrowserDiagram {
     virtual void renumber(int phase);
     static void open_all();
     static void import();
+    
+    static void compute_referenced_by(QList<BrowserNode> & l, BrowserNode *,
+				      const char * kc, char const * kr);
 };
 
 #endif
