@@ -1,6 +1,6 @@
 // *************************************************************************
 //
-// Copyleft 2004-2008 Bruno PAGES  .
+// Copyleft 2004-2009 Bruno PAGES  .
 //
 // This file is part of the BOUML Uml Toolkit.
 //
@@ -78,6 +78,10 @@ void ClassInstanceData::delete_it() {
 }
 
 void ClassInstanceData::edit() {
+  // to take into account deleted class instances being
+  // the value of relations
+  check_rels();
+  
   setName(browser_node->get_name());
   
   (new ClassInstanceDialog(this))->show();
