@@ -1,6 +1,6 @@
 // *************************************************************************
 //
-// Copyleft 2004-2008 Bruno PAGES  .
+// Copyleft 2004-2009 Bruno PAGES  .
 //
 // This file is part of the BOUML Uml Toolkit.
 //
@@ -28,6 +28,8 @@
 
 
 #include "CodMsgSupport.h"
+#include "BrowserNode.h"
+#include "OperationData.h"
 
 CodMsgSupport::~CodMsgSupport(){
 }
@@ -36,3 +38,14 @@ void CodMsgSupport::delete_it(ColMsgList & top) {
   while (msgs.getFirst() != 0)
     msgs.getFirst()->delete_it(FALSE, top);	// remove msg
 }
+
+bool CodMsgSupport::supports(BrowserNode * bn) {
+  BasicData * data = bn->get_data();
+  ColMsg * msg;
+  
+  for (msg = msgs.first(); msg != 0; msg = msgs.next())
+    if ((BasicData *) msg->get_operation() == data)
+      return TRUE;
+  
+  return FALSE;
+}
