@@ -1,6 +1,6 @@
 // *************************************************************************
 //
-// Copyleft 2004-2008 Bruno PAGES  .
+// Copyleft 2004-2009 Bruno PAGES  .
 //
 // This file is part of the BOUML Uml Toolkit.
 //
@@ -28,6 +28,8 @@
 
 #include "DiagramView.h"
 
+template <class K> class QPtrDict;
+
 class ComponentDiagramWindow;
 class DiagramItem;
 class UmlCanvas;
@@ -39,13 +41,17 @@ class ComponentDiagramView : public DiagramView {
     ComponentDiagramView(QWidget * parent, UmlCanvas * canvas, int id);
   
     virtual void menu(const QPoint&);
+    virtual void add_related_elements(DiagramItem *, const char * what,
+				      bool inh, bool assoc);
     virtual void read(char *, char * k);
     virtual void save(QTextStream & st, QString & warning, bool copy) const;
     
   private:
     ComponentDiagramWindow * window() {
       return (ComponentDiagramWindow *) parent();
-    };
+    }
+    void add_marked_elements(const QPoint& p,
+			     QPtrDict<DiagramItem> & drawn);
   
   protected:
     virtual void contentsMousePressEvent(QMouseEvent *);
