@@ -1,6 +1,6 @@
 // *************************************************************************
 //
-// Copyleft 2004-2008 Bruno PAGES  .
+// Copyleft 2004-2009 Bruno PAGES  .
 //
 // This file is part of the BOUML Uml Toolkit.
 //
@@ -221,6 +221,10 @@ bool IconCanvas::copyable() const {
   return selected();
 }
 
+bool IconCanvas::represents(BrowserNode * bn) {
+  return (bn == browser_node);
+}
+
 void IconCanvas::save(QTextStream & st, bool, QString & warning) const {
   nl_indent(st);
   st << "iconcanvas " << get_ident() << ' ';
