@@ -1,6 +1,6 @@
 // *************************************************************************
 //
-// Copyleft 2004-2008 Bruno PAGES  .
+// Copyleft 2004-2009 Bruno PAGES  .
 //
 // This file is part of the BOUML Uml Toolkit.
 //
@@ -861,6 +861,34 @@ double SdDurationCanvas::getZ() const {
   return z();
 }
 
+void SdDurationCanvas::propag_visible(QList<SdDurationCanvas> & l, bool y)
+{
+  QListIterator<SdDurationCanvas> itd(l);
+  SdDurationCanvas * d;
+  
+  for ( ; (d = itd.current()) != 0; ++itd ) {
+    d->QCanvasItem::setVisible(y);
+    
+    QListIterator<SdMsgBaseCanvas> itm(d->msgs);
+    
+    for ( ; itm.current(); ++itm )
+      itm.current()->QCanvasItem::setVisible(y);
+    
+    propag_visible(d->durations, y);
+  }
+}
+
+void SdDurationCanvas::propag_dz(QList<SdDurationCanvas> & l, double dz)
+{
+  QListIterator<SdDurationCanvas> it(l);
+  SdDurationCanvas * d;
+  
+  for ( ; (d = it.current()) != 0; ++it ) {
+    d->QCanvasItem::setZ(d->z() + dz);
+    propag_dz(d->durations, dz);
+  }
+}
+
 void SdDurationCanvas::save_sub(QTextStream & st) const {
   nl_indent(st);
   st << "overlappingdurationcanvas " << get_ident();
@@ -1113,9 +1141,9 @@ void SdDurationCanvas::send(ToolCom * com, int id) const {
   }
   
   if (!isreturn)
-    SdMsgBaseCanvas::send_implicit_return(com, id,
-					  (unsigned) x() + width(),
-					  (unsigned) y() + height());
+    SdMsgBaseCanvas::send(com, id, (unsigned) x() + width(),
+			  (unsigned) y() + height(),
+			  anImplicitReturn, "", "");
   
   QListIterator<SdDurationCanvas> itd(durations);
   
