@@ -1,6 +1,6 @@
 // *************************************************************************
 //
-// Copyleft 2004-2008 Bruno PAGES  .
+// Copyleft 2004-2009 Bruno PAGES  .
 //
 // This file is part of the BOUML Uml Toolkit.
 //
@@ -123,6 +123,9 @@ class SdDurationCanvas : public QObject, public DiagramCanvas, public SdDuration
     unsigned count_msg() const;
     void send(ToolCom * com, int id) const;
     
+    static void propag_visible(QList<SdDurationCanvas> &, bool y);
+    static void propag_dz(QList<SdDurationCanvas> &, double dz);
+    
   private slots:
     void modified();
 };
