@@ -1,6 +1,6 @@
 // *************************************************************************
 //
-// Copyleft 2004-2008 Bruno PAGES  .
+// Copyleft 2004-2009 Bruno PAGES  .
 //
 // This file is part of the BOUML Uml Toolkit.
 //
@@ -33,11 +33,15 @@
 #include <qcombobox.h> 
 #include <qpushbutton.h> 
 #include <qcheckbox.h> 
-#include <qbuttongroup.h> 
+#include <qbuttongroup.h>  
+#include <qpopupmenu.h> 
+#include <qcursor.h> 
 
 #include "ActivityDialog.h"
 #include "ActivityData.h"
 #include "BrowserActivity.h"
+#include "BrowserOperation.h"
+#include "BrowserView.h"
 #include "KeyValueTable.h"
 #include "UmlWindow.h"
 #include "DialogUtil.h"
@@ -90,6 +94,27 @@ ActivityDialog::ActivityDialog(ActivityData * d)
   QSizePolicy sp = edstereotype->sizePolicy();
   sp.setHorData(QSizePolicy::Expanding);
   edstereotype->setSizePolicy(sp);
+    
+  connect(new SmallPushButton("specification :", grid), SIGNAL(clicked()),
+	  this, SLOT(menu_specification()));    
+  edspecification = new QComboBox(FALSE, grid);
+  edspecification->setSizePolicy(sp);
+  if (visit) {
+    if (activity->get_specification() == 0)
+      edspecification->insertItem("");
+    else
+      edspecification->insertItem(activity->get_specification()->full_name(TRUE));
+  }
+  else {
+    edspecification->insertItem("");
+    edspecification->setAutoCompletion(TRUE);
+    BrowserOperation::instances(opers);
+    opers.full_names(list);
+    edspecification->insertStringList(list);
+    edspecification->setCurrentItem((activity->get_specification() == 0)
+				    ? 0
+				    : opers.findRef(activity->get_specification()) + 1);
+  }
   
   new QLabel(grid);
   QButtonGroup * bg = 
@@ -148,6 +173,60 @@ void ActivityDialog::polish() {
   UmlDesktop::limitsize_center(this, previous_size, 0.8, 0.8);
 }
 
+void ActivityDialog::menu_specification() {
+  QPopupMenu m(0);
+
+  m.insertItem("Choose", -1);
+  m.insertSeparator();
+  
+  int index = list.findIndex(edspecification->currentText().stripWhiteSpace());
+  
+  if (index != -1)
+    m.insertItem("Select in browser", 0);
+  
+  BrowserNode * bn = 0;
+  
+  if (! visit) {
+    bn = BrowserView::selected_item();
+    
+    if ((bn != 0) && (bn->get_type() == UmlOperation) && !bn->deletedp())
+      m.insertItem("Choose operation selected in browser", 1);
+    else
+      bn = 0;
+  }
+  
+  if ((index != -1) || (bn != 0)) {
+    switch (m.exec(QCursor::pos())) {
+    case 0:
+      opers.at(index)->select_in_browser();
+      break;
+    case 1:
+      {
+	QString s = bn->full_name(TRUE);
+	
+	if ((index = list.findIndex(s)) == -1) {
+	  // new operation, may be created through an other dialog
+	  index = 0;
+	  QStringList::Iterator iter = list.begin();
+	  QStringList::Iterator iter_end = list.end();
+	  
+	  while ((iter != iter_end) && (*iter < s)) {
+	    ++iter;
+	    index += 1;
+	  }
+	  opers.insert((unsigned) index, bn);
+	  list.insert(iter, s);
+	  edspecification->insertItem(s, index + 1);
+	}
+      }
+      edspecification->setCurrentItem(index + 1);
+      break;
+    default:
+      break;
+    }
+  }
+}
+
 ActivityDialog::~ActivityDialog() {
   activity->browser_node->edit_end();
   previous_size = size();
@@ -196,7 +275,7 @@ void ActivityDialog::post_edit_description(ActivityDialog * d, QString s)
 }
 
 void ActivityDialog::accept() {
-  if (!check_edits(edits))
+  if (!check_edits(edits) || !kvtable->check_unique())
     return;
     
   BrowserNode * bn = activity->browser_node;
@@ -212,6 +291,11 @@ void ActivityDialog::accept() {
     bn->set_name(s);
 
     bool newst = activity->set_stereotype(fromUnicode(edstereotype->currentText().stripWhiteSpace()));
+    int index = list.findIndex(edspecification->currentText().stripWhiteSpace());
+    
+    activity->set_specification((index != -1)
+				? (BrowserOperation *) opers.at(index)
+				: 0);
     
     activity->read_only = readonly_cb->isChecked();
     activity->single_execution = singlexec_cb->isChecked();
