@@ -1,6 +1,6 @@
 // *************************************************************************
 //
-// Copyleft 2004-2008 Bruno PAGES  .
+// Copyleft 2004-2009 Bruno PAGES  .
 //
 // This file is part of the BOUML Uml Toolkit.
 //
@@ -1735,7 +1735,7 @@ void ArtifactDialog::stereotypeFilterActivated(const QString & st) {
 }
 
 void ArtifactDialog::accept() {
-  if (!check_edits(edits))
+  if (!check_edits(edits) || !kvtable->check_unique())
     return;
     
   QString s = edname->text().stripWhiteSpace();
