@@ -1,6 +1,6 @@
 // *************************************************************************
 //
-// Copyleft 2004-2008 Bruno PAGES  .
+// Copyleft 2004-2009 Bruno PAGES  .
 //
 // This file is part of the BOUML Uml Toolkit.
 //
@@ -191,7 +191,7 @@ void ClassViewDialog::post_edit_description(ClassViewDialog * d, QString s) {
 }
 
 void ClassViewDialog::accept() {
-  if (!check_edits(edits))
+  if (!check_edits(edits) || !kvtable->check_unique())
     return;
     
   QString s = edname->text().stripWhiteSpace();
