@@ -1,6 +1,6 @@
 // *************************************************************************
 //
-// Copyleft 2004-2008 Bruno PAGES  .
+// Copyleft 2004-2009 Bruno PAGES  .
 //
 // This file is part of the BOUML Uml Toolkit.
 //
@@ -415,7 +415,7 @@ void ComponentDialog::post_edit_description(ComponentDialog * d, QString s) {
 }
 
 void ComponentDialog::accept() {
-  if (!check_edits(edits))
+  if (!check_edits(edits) || !kvtable->check_unique())
     return;
     
   QString s = edname->text().stripWhiteSpace();
