@@ -1,6 +1,6 @@
 // *************************************************************************
 //
-// Copyleft 2004-2008 Bruno PAGES  .
+// Copyleft 2004-2009 Bruno PAGES  .
 //
 // This file is part of the BOUML Uml Toolkit.
 //
@@ -961,6 +961,11 @@ void GenerationSettingsDialog::init_python1() {
   edpython_enum_decl->setText(GenerationSettings::python_enum_decl);
   edpython_enum_decl->setFont(font);
 
+  new QLabel("Enumeration item \ndefault definition :", grid);
+  edpython_enum_item_decl = new MultiLineEdit(grid);
+  edpython_enum_item_decl->setText(GenerationSettings::python_enum_item_decl);
+  edpython_enum_item_decl->setFont(font);
+
   new QLabel("Attribute \ndefault \ndeclaration :", grid);
 
   QGrid * grid2 = new QGrid(2, grid);
@@ -1029,14 +1034,6 @@ void GenerationSettingsDialog::init_python2() {
   new QLabel(grid);
   new QLabel(grid);
 
-  new QLabel("Enumeration item \ndefault definition :", grid);
-  edpython_enum_item_decl = new MultiLineEdit(grid);
-  edpython_enum_item_decl->setText(GenerationSettings::python_enum_item_decl);
-  edpython_enum_item_decl->setFont(font);
-
-  new QLabel(grid);
-  new QLabel(grid);
-
   new QLabel("Get operation\ndefault definition : ", grid);
   htab = new QHBox(grid);
   htab->setMargin(3);
@@ -1075,6 +1072,11 @@ void GenerationSettingsDialog::init_python2() {
   edpython_oper_def->setText(GenerationSettings::python_oper_def);
   edpython_oper_def->setFont(font);
   
+  new QLabel("Operation __init__\ndefault definition :", grid);
+  edpython_initoper_def = new MultiLineEdit(grid);
+  edpython_initoper_def->setText(GenerationSettings::python_initoper_def);
+  edpython_initoper_def->setFont(font);
+  
   addTab(grid, "Python[2]");
   
   if (!GenerationSettings::python_get_default_defs())
@@ -1839,6 +1841,7 @@ void GenerationSettingsDialog::accept() {
     GenerationSettings::python_attr_decl[0] = edpython_attr_decl[0]->text();
     GenerationSettings::python_attr_decl[1] = edpython_attr_decl[1]->text();
     GenerationSettings::python_oper_def = edpython_oper_def->text();
+    GenerationSettings::python_initoper_def = edpython_initoper_def->text();
     
     GenerationSettings::idl_interface_decl = edidl_interface_decl->text();
     GenerationSettings::idl_valuetype_decl = edidl_valuetype_decl->text();
