@@ -1,6 +1,6 @@
 // *************************************************************************
 //
-// Copyleft 2004-2008 Bruno PAGES  .
+// Copyleft 2004-2009 Bruno PAGES  .
 //
 // This file is part of the BOUML Uml Toolkit.
 //
@@ -164,6 +164,7 @@ class GenerationSettings {
     static SharedStr python_enum_item_decl;
     static SharedStr python_rel_decl[2/*relation kind*/][2/*multiplicity*/];
     static SharedStr python_oper_def;
+    static SharedStr python_initoper_def;
     static SharedStr python_get_name;
     static SharedStr python_set_name;
     
@@ -332,6 +333,7 @@ class GenerationSettings {
     static const char * python_default_enum_item_decl() { return python_enum_item_decl; };
     static const char * python_default_rel_decl(UmlCode rel, const QString & mult);
     static const char * python_default_oper_def() { return python_oper_def; };
+    static const char * python_default_initoper_def() { return python_initoper_def; };
     static const char * python_default_get_name() { return python_get_name; };
     static const char * python_default_set_name() { return python_set_name; };
 
