@@ -26,6 +26,7 @@ extern QPixmap * hubButton;
 extern QPixmap * generalisationButton;
 extern QPixmap * associationButton;
 extern QPixmap * directionalAssociationButton;
+extern QPixmap * directionalAssociationExtensionButton;
 extern QPixmap * aggregationButton;
 extern QPixmap * aggregationByValueButton;
 extern QPixmap * directionalAggregationButton;
@@ -62,6 +63,7 @@ extern QPixmap * signaloutButton;
 extern QPixmap * activityButton;
 extern QPixmap * interruptibleactivityregionButton;
 extern QPixmap * activityactionButton;
+extern QPixmap * activitypartitionButton;
 extern QPixmap * decisionButton;
 extern QPixmap * mergeButton;
 extern QPixmap * pinButton;
@@ -194,6 +196,9 @@ extern QPixmap * DeletedActivityIcon;
 extern QPixmap * ActivityActionIcon;
 extern QPixmap * ActivityActionMarkedIcon;
 extern QPixmap * DeletedActivityActionIcon;
+extern QPixmap * ActivityPartitionIcon;
+extern QPixmap * ActivityPartitionMarkedIcon;
+extern QPixmap * DeletedActivityPartitionIcon;
 extern QPixmap * ActivityDiagramIcon;
 extern QPixmap * DeletedActivityDiagramIcon;
 extern QPixmap * ParameterIcon;
